@@ -50,6 +50,7 @@
 #include "unix/SignalHandlers.hxx"
 #include "system/FatalError.hxx"
 #include "thread/Slack.hxx"
+#include "net/Init.hxx"
 #include "lib/icu/Init.hxx"
 #include "config/ConfigGlobal.hxx"
 #include "config/Param.hxx"
@@ -106,15 +107,6 @@
 #include <locale.h>
 #endif
 
-#ifdef WIN32
-#include <winsock2.h>
-#include <ws2tcpip.h>
-#endif
-
-#ifdef __BLOCKS__
-#include <dispatch/dispatch.h>
-#endif
-
 #include <limits.h>
 
 static constexpr size_t KILOBYTE = 1024;
@@ -284,25 +276,6 @@ glue_state_file_init()
 	instance->state_file->Read();
 }
 
-/**
- * Windows-only initialization of the Winsock2 library.
- */
-static void winsock_init(void)
-{
-#ifdef WIN32
-	WSADATA sockinfo;
-
-	int retval = WSAStartup(MAKEWORD(2, 2), &sockinfo);
-	if(retval != 0)
-		FormatFatalError("Attempt to open Winsock2 failed; error code %d",
-				 retval);
-
-	if (LOBYTE(sockinfo.wVersion) != 2)
-		FatalError("We use Winsock2 but your version is either too new "
-			   "or old; please install Winsock 2.x");
-#endif
-}
-
 /**
  * Initialize the decoder and player core, including the music pipe.
  */
@@ -418,7 +391,7 @@ Instance::OnIdle(unsigned flags)
 
 int main(int argc, char *argv[])
 {
-#ifdef WIN32
+#ifdef _WIN32
 	return win32_main(argc, argv);
 #else
 	return mpd_main(argc, argv);
@@ -451,7 +424,8 @@ try {
 
 	IcuInit();
 
-	winsock_init();
+	const ScopeNetInit net_init;
+
 	io_thread_init();
 	config_global_init();
 
@@ -505,21 +479,8 @@ try {
 	daemonize_begin(options.daemon);
 #endif
 
-#ifdef __BLOCKS__
-	/* Runs the OS X native event loop in the main thread, and runs
-	   the rest of mpd_main on a new thread. This lets CoreAudio receive
-	   route change notifications (e.g. plugging or unplugging headphones).
-	   All hardware output on OS X ultimately uses CoreAudio internally.
-	   This must be run after forking; if dispatch is called before forking,
-	   the child process will have a broken internal dispatch state. */
-	dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
-		exit(mpd_main_after_fork(config));
-	});
-	dispatch_main();
-	return EXIT_FAILURE; // unreachable, because dispatch_main never returns
-#else
 	return mpd_main_after_fork(config);
-#endif
+
 } catch (const std::exception &e) {
 	LogError(e);
 	return EXIT_FAILURE;
@@ -615,7 +576,7 @@ try {
 	   playlist_state_restore() */
 	instance->partition->pc.LockUpdateAudio();
 
-#ifdef WIN32
+#ifdef _WIN32
 	win32_app_started();
 #endif
 
@@ -630,7 +591,7 @@ try {
 	/* run the main loop */
 	instance->event_loop.Run();
 
-#ifdef WIN32
+#ifdef _WIN32
 	win32_app_stopping();
 #endif
 
@@ -702,10 +663,6 @@ try {
 	daemonize_finish();
 #endif
 
-#ifdef WIN32
-	WSACleanup();
-#endif
-
 	IcuFinish();
 
 	log_deinit();
