@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include "tag/TagBuilder.hxx"
 #include "tag/TagTable.hxx"
 #include "util/NumberParser.hxx"
+#include "util/StringView.hxx"
 
 #include <algorithm>
 #include <string>
@@ -36,23 +37,13 @@ UPnPDirContent::~UPnPDirContent()
 	/* this destructor exists here just so it won't get inlined */
 }
 
-gcc_pure gcc_nonnull_all
-static bool
-CompareStringLiteral(const char *literal, const char *value, size_t length)
-{
-	return length == strlen(literal) &&
-		memcmp(literal, value, length) == 0;
-}
-
 gcc_pure
 static UPnPDirObject::ItemClass
-ParseItemClass(const char *name, size_t length)
+ParseItemClass(StringView name) noexcept
 {
-	if (CompareStringLiteral("object.item.audioItem.musicTrack",
-				 name, length))
+	if (name.EqualsLiteral("object.item.audioItem.musicTrack"))
 		return UPnPDirObject::ItemClass::MUSIC;
-	else if (CompareStringLiteral("object.item.playlistItem",
-				      name, length))
+	else if (name.EqualsLiteral("object.item.playlistItem"))
 		return UPnPDirObject::ItemClass::PLAYLIST;
 	else
 		return UPnPDirObject::ItemClass::UNKNOWN;
@@ -60,7 +51,7 @@ ParseItemClass(const char *name, size_t length)
 
 gcc_pure
 static SignedSongTime
-ParseDuration(const char *duration)
+ParseDuration(const char *duration) noexcept
 {
 	char *endptr;
 
@@ -89,18 +80,18 @@ ParseDuration(const char *duration)
  * this. Twonky returns directory names (titles) like 'Artist/Album'.
  */
 gcc_pure
-static std::string
-titleToPathElt(std::string &&s)
+static std::string &&
+TitleToPathSegment(std::string &&s) noexcept
 {
 	std::replace(s.begin(), s.end(), '/', '_');
-	return s;
+	return std::move(s);
 }
 
 /**
  * An XML parser which builds directory contents from DIDL lite input.
  */
 class UPnPDirParser final : public CommonExpatParser {
-	UPnPDirContent &m_dir;
+	UPnPDirContent &directory;
 
 	enum {
 		NONE,
@@ -120,22 +111,22 @@ class UPnPDirParser final : public CommonExpatParser {
 	 */
 	std::string value;
 
-	UPnPDirObject m_tobj;
+	UPnPDirObject object;
 	TagBuilder tag;
 
 public:
-	UPnPDirParser(UPnPDirContent& dir)
-		:m_dir(dir),
+	UPnPDirParser(UPnPDirContent &_directory)
+		:directory(_directory),
 		 state(NONE),
 		 tag_type(TAG_NUM_OF_ITEM_TYPES)
 	{
-		m_tobj.clear();
+		object.Clear();
 	}
 
 protected:
-	virtual void StartElement(const XML_Char *name, const XML_Char **attrs)
+	void StartElement(const XML_Char *name, const XML_Char **attrs) override
 	{
-		if (m_tobj.type != UPnPDirObject::Type::UNKNOWN &&
+		if (object.type != UPnPDirObject::Type::UNKNOWN &&
 		    tag_type == TAG_NUM_OF_ITEM_TYPES) {
 			tag_type = tag_table_lookup(upnp_tags, name);
 			if (tag_type != TAG_NUM_OF_ITEM_TYPES)
@@ -147,31 +138,31 @@ protected:
 		switch (name[0]) {
 		case 'c':
 			if (!strcmp(name, "container")) {
-				m_tobj.clear();
-				m_tobj.type = UPnPDirObject::Type::CONTAINER;
+				object.Clear();
+				object.type = UPnPDirObject::Type::CONTAINER;
 
 				const char *id = GetAttribute(attrs, "id");
 				if (id != nullptr)
-					m_tobj.m_id = id;
+					object.id = id;
 
 				const char *pid = GetAttribute(attrs, "parentID");
 				if (pid != nullptr)
-					m_tobj.m_pid = pid;
+					object.parent_id = pid;
 			}
 			break;
 
 		case 'i':
 			if (!strcmp(name, "item")) {
-				m_tobj.clear();
-				m_tobj.type = UPnPDirObject::Type::ITEM;
+				object.Clear();
+				object.type = UPnPDirObject::Type::ITEM;
 
 				const char *id = GetAttribute(attrs, "id");
 				if (id != nullptr)
-					m_tobj.m_id = id;
+					object.id = id;
 
 				const char *pid = GetAttribute(attrs, "parentID");
 				if (pid != nullptr)
-					m_tobj.m_pid = pid;
+					object.parent_id = pid;
 			}
 			break;
 
@@ -197,25 +188,15 @@ protected:
 		}
 	}
 
-	bool checkobjok() {
-		if (m_tobj.m_id.empty() || m_tobj.m_pid.empty() ||
-		    m_tobj.name.empty() ||
-		    (m_tobj.type == UPnPDirObject::Type::ITEM &&
-		     m_tobj.item_class == UPnPDirObject::ItemClass::UNKNOWN))
-			return false;
-
-		return true;
-	}
-
-	virtual void EndElement(const XML_Char *name)
+	void EndElement(const XML_Char *name) override
 	{
 		if (tag_type != TAG_NUM_OF_ITEM_TYPES) {
-			assert(m_tobj.type != UPnPDirObject::Type::UNKNOWN);
+			assert(object.type != UPnPDirObject::Type::UNKNOWN);
 
 			tag.AddItem(tag_type, value.c_str());
 
 			if (tag_type == TAG_TITLE)
-				m_tobj.name = titleToPathElt(std::move(value));
+				object.name = TitleToPathSegment(std::move(value));
 
 			value.clear();
 			tag_type = TAG_NUM_OF_ITEM_TYPES;
@@ -223,18 +204,18 @@ protected:
 		}
 
 		if ((!strcmp(name, "container") || !strcmp(name, "item")) &&
-		    checkobjok()) {
-			tag.Commit(m_tobj.tag);
-			m_dir.objects.emplace_back(std::move(m_tobj));
+		    object.Check()) {
+			tag.Commit(object.tag);
+			directory.objects.emplace_back(std::move(object));
 		}
 
 		state = NONE;
 	}
 
-	virtual void CharacterData(const XML_Char *s, int len)
+	void CharacterData(const XML_Char *s, int len) override
 	{
 		if (tag_type != TAG_NUM_OF_ITEM_TYPES) {
-			assert(m_tobj.type != UPnPDirObject::Type::UNKNOWN);
+			assert(object.type != UPnPDirObject::Type::UNKNOWN);
 
 			value.append(s, len);
 			return;
@@ -245,19 +226,19 @@ protected:
 			break;
 
 		case RES:
-			m_tobj.url.assign(s, len);
+			object.url.assign(s, len);
 			break;
 
 		case CLASS:
-			m_tobj.item_class = ParseItemClass(s, len);
+			object.item_class = ParseItemClass(StringView(s, len));
 			break;
 		}
 	}
 };
 
-bool
-UPnPDirContent::parse(const char *input, Error &error)
+void
+UPnPDirContent::Parse(const char *input)
 {
 	UPnPDirParser parser(*this);
-	return parser.Parse(input, strlen(input), true, error);
+	parser.Parse(input, strlen(input), true);
 }
