@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,18 +28,18 @@
 #include "../DecoderAPI.hxx"
 #include "input/InputStream.hxx"
 #include "tag/TagId3.hxx"
-#include "util/Error.hxx"
-#include "util/Alloc.hxx"
+
+#ifdef ENABLE_ID3TAG
+#include <id3tag.h>
+#endif
+
+#include <stdexcept>
 
 #include <string.h>
 #include <stdlib.h>
 
-#ifdef HAVE_ID3TAG
-#include <id3tag.h>
-#endif
-
 bool
-DsdId::Equals(const char *s) const
+DsdId::Equals(const char *s) const noexcept
 {
 	assert(s != nullptr);
 	assert(strlen(s) == sizeof(value));
@@ -48,44 +48,54 @@ DsdId::Equals(const char *s) const
 }
 
 /**
- * Skip the #input_stream to the specified offset.
+ * Skip the #InputStream to the specified offset.
  */
 bool
-dsdlib_skip_to(Decoder *decoder, InputStream &is,
+dsdlib_skip_to(DecoderClient *client, InputStream &is,
 	       offset_type offset)
 {
-	if (is.IsSeekable())
-		return is.LockSeek(offset, IgnoreError());
+	if (is.IsSeekable()) {
+		try {
+			is.LockSeek(offset);
+		} catch (const std::runtime_error &) {
+			return false;
+		}
+	}
 
 	if (is.GetOffset() > offset)
 		return false;
 
-	return dsdlib_skip(decoder, is, offset - is.GetOffset());
+	return dsdlib_skip(client, is, offset - is.GetOffset());
 }
 
 /**
- * Skip some bytes from the #input_stream.
+ * Skip some bytes from the #InputStream.
  */
 bool
-dsdlib_skip(Decoder *decoder, InputStream &is,
+dsdlib_skip(DecoderClient *client, InputStream &is,
 	    offset_type delta)
 {
 	if (delta == 0)
 		return true;
 
-	if (is.IsSeekable())
-		return is.LockSeek(is.GetOffset() + delta, IgnoreError());
+	if (is.IsSeekable()) {
+		try {
+			is.LockSeek(is.GetOffset() + delta);
+		} catch (const std::runtime_error &) {
+			return false;
+		}
+	}
 
 	if (delta > 1024 * 1024)
 		/* don't skip more than one megabyte; it would be too
 		   expensive */
 		return false;
 
-	return decoder_skip(decoder, is, delta);
+	return decoder_skip(client, is, delta);
 }
 
 bool
-dsdlib_valid_freq(uint32_t samplefreq)
+dsdlib_valid_freq(uint32_t samplefreq) noexcept
 {
 	switch (samplefreq) {
 	case 2822400: /* DSD64, 64xFs, Fs = 44.100kHz */
@@ -103,30 +113,28 @@ dsdlib_valid_freq(uint32_t samplefreq)
 	}
 }
 
-#ifdef HAVE_ID3TAG
+#ifdef ENABLE_ID3TAG
 void
 dsdlib_tag_id3(InputStream &is,
-	       const struct tag_handler *handler,
-	       void *handler_ctx, int64_t tagoffset)
+	       const TagHandler &handler,
+	       void *handler_ctx, offset_type tagoffset)
 {
-	assert(tagoffset >= 0);
-
 	if (tagoffset == 0 || !is.KnownSize())
 		return;
 
+	/* Prevent broken files causing problems */
+	const auto size = is.GetSize();
+	if (tagoffset >= size)
+		return;
+
+	const auto count64 = size - tagoffset;
+	if (count64 < 10 || count64 > 4 * 1024 * 1024)
+		return;
+
 	if (!dsdlib_skip_to(nullptr, is, tagoffset))
 		return;
 
-	/* Prevent broken files causing problems */
-	const auto size = is.GetSize();
-	const auto offset = is.GetOffset();
-	if (offset >= size)
-		return;
-
-	const id3_length_t count = size - offset;
-
-	if (count < 10 || count > 1024 * 1024)
-		return;
+	const id3_length_t count = count64;
 
 	id3_byte_t *const id3_buf = new id3_byte_t[count];
 	if (id3_buf == nullptr)
