@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,9 @@
 #include "config.h"
 #include "InputStream.hxx"
 #include "thread/Cond.hxx"
-#include "util/StringUtil.hxx"
+#include "util/ASCII.hxx"
+
+#include <stdexcept>
 
 #include <assert.h>
 
@@ -28,10 +30,9 @@ InputStream::~InputStream()
 {
 }
 
-bool
-InputStream::Check(gcc_unused Error &error)
+void
+InputStream::Check()
 {
-	return true;
 }
 
 void
@@ -63,7 +64,7 @@ InputStream::WaitReady()
 void
 InputStream::LockWaitReady()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	WaitReady();
 }
 
@@ -74,30 +75,36 @@ InputStream::LockWaitReady()
  */
 gcc_pure
 static bool
-ExpensiveSeeking(const char *uri)
+ExpensiveSeeking(const char *uri) noexcept
 {
-	return StringStartsWith(uri, "http://") ||
-		StringStartsWith(uri, "https://");
+	return StringStartsWithCaseASCII(uri, "http://") ||
+		StringStartsWithCaseASCII(uri, "https://");
 }
 
 bool
-InputStream::CheapSeeking() const
+InputStream::CheapSeeking() const noexcept
 {
 	return IsSeekable() && !ExpensiveSeeking(uri.c_str());
 }
 
-bool
-InputStream::Seek(gcc_unused offset_type new_offset,
-		  gcc_unused Error &error)
+void
+InputStream::Seek(gcc_unused offset_type new_offset)
 {
-	return false;
+	throw std::runtime_error("Seeking is not implemented");
 }
 
-bool
-InputStream::LockSeek(offset_type _offset, Error &error)
+void
+InputStream::LockSeek(offset_type _offset)
 {
-	const ScopeLock protect(mutex);
-	return Seek(_offset, error);
+	const std::lock_guard<Mutex> protect(mutex);
+	Seek(_offset);
+}
+
+void
+InputStream::LockSkip(offset_type _offset)
+{
+	const std::lock_guard<Mutex> protect(mutex);
+	Skip(_offset);
 }
 
 Tag *
@@ -109,18 +116,18 @@ InputStream::ReadTag()
 Tag *
 InputStream::LockReadTag()
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	return ReadTag();
 }
 
 bool
-InputStream::IsAvailable()
+InputStream::IsAvailable() noexcept
 {
 	return true;
 }
 
 size_t
-InputStream::LockRead(void *ptr, size_t _size, Error &error)
+InputStream::LockRead(void *ptr, size_t _size)
 {
 #if !CLANG_CHECK_VERSION(3,6)
 	/* disabled on clang due to -Wtautological-pointer-compare */
@@ -128,14 +135,42 @@ InputStream::LockRead(void *ptr, size_t _size, Error &error)
 #endif
 	assert(_size > 0);
 
-	const ScopeLock protect(mutex);
-	return Read(ptr, _size, error);
+	const std::lock_guard<Mutex> protect(mutex);
+	return Read(ptr, _size);
+}
+
+void
+InputStream::ReadFull(void *_ptr, size_t _size)
+{
+	uint8_t *ptr = (uint8_t *)_ptr;
+
+	size_t nbytes_total = 0;
+	while (_size > 0) {
+		size_t nbytes = Read(ptr + nbytes_total, _size);
+		if (nbytes == 0)
+			throw std::runtime_error("Unexpected end of file");
+
+		nbytes_total += nbytes;
+		_size -= nbytes;
+	}
+}
+
+void
+InputStream::LockReadFull(void *ptr, size_t _size)
+{
+#if !CLANG_CHECK_VERSION(3,6)
+	/* disabled on clang due to -Wtautological-pointer-compare */
+	assert(ptr != nullptr);
+#endif
+	assert(_size > 0);
+
+	const std::lock_guard<Mutex> protect(mutex);
+	ReadFull(ptr, _size);
 }
 
 bool
-InputStream::LockIsEOF()
+InputStream::LockIsEOF() noexcept
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	return IsEOF();
 }
-
