@@ -19,6 +19,7 @@
 
 #include "config.h"
 #include "CurlInputPlugin.hxx"
+#include "lib/curl/Error.hxx"
 #include "lib/curl/Easy.hxx"
 #include "lib/curl/Global.hxx"
 #include "lib/curl/Request.hxx"
@@ -34,12 +35,15 @@
 #include "IOThread.hxx"
 #include "util/ASCII.hxx"
 #include "util/StringUtil.hxx"
+#include "util/StringFormat.hxx"
 #include "util/NumberParser.hxx"
 #include "util/RuntimeError.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 #include "PluginUnavailable.hxx"
 
+#include <cinttypes>
+
 #include <assert.h>
 #include <string.h>
 
@@ -185,7 +189,9 @@ CurlInputStream::OnHeaders(unsigned status,
 	assert(!postponed_exception);
 
 	if (status < 200 || status >= 300)
-		throw FormatRuntimeError("got HTTP status %ld", status);
+		throw HttpStatusError(status,
+				      StringFormat<40>("got HTTP status %u",
+						       status).c_str());
 
 	const std::lock_guard<Mutex> protect(mutex);
 
@@ -371,13 +377,10 @@ CurlInputStream::InitEasy()
 	if (proxy_port > 0)
 		request->SetOption(CURLOPT_PROXYPORT, (long)proxy_port);
 
-	if (proxy_user != nullptr && proxy_password != nullptr) {
-		char proxy_auth_str[1024];
-		snprintf(proxy_auth_str, sizeof(proxy_auth_str),
-			 "%s:%s",
-			 proxy_user, proxy_password);
-		request->SetOption(CURLOPT_PROXYUSERPWD, proxy_auth_str);
-	}
+	if (proxy_user != nullptr && proxy_password != nullptr)
+		request->SetOption(CURLOPT_PROXYUSERPWD,
+				   StringFormat<1024>("%s:%s", proxy_user,
+						      proxy_password).c_str());
 
 	request->SetOption(CURLOPT_SSL_VERIFYPEER, verify_peer ? 1l : 0l);
 	request->SetOption(CURLOPT_SSL_VERIFYHOST, verify_host ? 2l : 0l);
@@ -414,16 +417,10 @@ CurlInputStream::SeekInternal(offset_type new_offset)
 
 	/* send the "Range" header */
 
-	if (offset > 0) {
-		char range[32];
-#ifdef _WIN32
-		// TODO: what can we use on Windows to format 64 bit?
-		sprintf(range, "%lu-", (long)offset);
-#else
-		sprintf(range, "%llu-", (unsigned long long)offset);
-#endif
-		request->SetOption(CURLOPT_RANGE, range);
-	}
+	if (offset > 0)
+		request->SetOption(CURLOPT_RANGE,
+				   StringFormat<40>("%" PRIoffset "-",
+						    offset).c_str());
 
 	StartRequest();
 }
@@ -461,8 +458,8 @@ CurlInputStream::Open(const char *url, Mutex &mutex, Cond &cond)
 static InputStream *
 input_curl_open(const char *url, Mutex &mutex, Cond &cond)
 {
-	if (strncmp(url, "http://", 7) != 0 &&
-	    strncmp(url, "https://", 8) != 0)
+	if (!StringStartsWithCaseASCII(url, "http://") &&
+	    !StringStartsWithCaseASCII(url, "https://"))
 		return nullptr;
 
 	return CurlInputStream::Open(url, mutex, cond);
