@@ -33,6 +33,9 @@ FfmpegInit()
 {
 	av_log_set_callback(FfmpegLogCallback);
 
+#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58, 9, 100)
+	/* deprecated as of FFmpeg 4.0 */
 	av_register_all();
+#endif
 }
 
